unit Dropbox;

// Get Dropbox info, state and file/folder status
// Port of the the python script by Steve H:
// http://wiki.dropbox.com/DropboxAddons/PythonScriptToGetFileOrFolderStatusInWindows

(* Delphi port by Manuel Krber <manuel.kroeber@googlemail.com>
 * Tested with Delphi 2010 Professional only. Revision 01/2011-08
 * https://bitbucket.org/h4ndy/dropboxdelphi

<License>
This port is Copyright (c) 2011, Manuel Krber
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form *ARE NOT REQUIRED* to reproduce the above
   copyright notice, this list of conditions and the following disclaimer in the
   documentation or other materials provided with the distribution.
   But it would be nice to do so.
3. Neither the name of the author nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MANUEL KRBER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MANUEL KRBER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
</License>

 ******************************************************************************)

interface

uses
  Windows, Classes, SysUtils;

type
  TDropboxStatus = (
    dsPipeError, dsNotRunning, dsNotInDropbox,
    dsUpToDate, dsSyncInProgress, dsSyncProblem
  );

  // Total size: 16 byte
  TDropboxPipeRequestStruct = packed record
    RequestInfo: Cardinal;
    ProcessId: Cardinal;
    ThreadId: Cardinal;
    RequestType: Cardinal;
  End;

  // Total size: 540 byte
  TDropboxPipeRequestMessage = packed record
    RequestStruct: TDropboxPipeRequestStruct;
    Path: array[0..523] of AnsiChar;
  end;

  TDropbox = class
  public
    class function GetStatus(const Filename: string): TDropboxStatus;
    class function StatusToStr(const DropboxStatus: TDropboxStatus): string;
  end;

implementation


{ TDropbox }

function ProcessIdToSessionId(const InProcessId: Cardinal;
  out OutSessionId: Cardinal): Boolean;
type
  TProcessIdToSessionId =
    function(dwProcessId: DWORD; pSessionId: DWORD): BOOL; stdcall;
var
  ProcessIdToSessionId: TProcessIdToSessionId;
  Lib : THandle;
  pSessionId : DWord;
begin
  Result := False;
  Lib := GetModuleHandle('kernel32');
  if Lib <> 0 then
  begin
    ProcessIdToSessionId := GetProcAddress(Lib, 'ProcessIdToSessionId');
    if Assigned(ProcessIdToSessionId) then
    begin
      Result := ProcessIdToSessionId(InProcessId, DWORD(@pSessionId));
      OutSessionId := pSessionId;
    end;
  end;
end;

class function TDropbox.GetStatus(const Filename: UnicodeString): TDropboxStatus;

  function CapValue(const InVal, MaxVal: Integer): Integer;
  begin
    if InVal > MaxVal then
      Result := MaxVal
    else
      Result := InVal;
  end;

var
  ProcessId, SessionId: Cardinal;
  PipeName: string;
  PipeRequest: TDropboxPipeRequestMessage;
  Response: array of Byte;
  ResultStr: AnsiString;
  BytesRead: Cardinal;
begin
  Result := dsPipeError;

  ProcessId := GetCurrentProcessId;

  if ProcessIdToSessionId(ProcessId, SessionId) then
  begin
    PipeName := '\\.\pipe\DropboxPipe_' + IntToStr(SessionId);

    PipeRequest.RequestStruct.RequestInfo := $3048302;
    PipeRequest.RequestStruct.ProcessId := ProcessId;
    PipeRequest.RequestStruct.ThreadId := GetCurrentThreadId;
    PipeRequest.RequestStruct.RequestType := 1;

    ZeroMemory(@PipeRequest.Path, SizeOf(PipeRequest.Path));
    CopyMemory(
      @PipeRequest.Path[0], @Filename[1], CapValue(Length(Filename)*SizeOf(Char), 524)
    );

    SetLength(Response, 16382);

    if CallNamedPipe(PChar(PipeName), @PipeRequest, SizeOf(PipeRequest),
      @Response[0], 16382, BytesRead, 1000) then
    begin
      SetLength(Response, BytesRead);

      SetString(ResultStr, PAnsiChar(@Response[4]), BytesRead-5);
      Result := TDropboxStatus(StrToIntDef(String(ResultStr), -2) + 2);
    end else
      RaiseLastOSError;

  end;
end;

class function TDropbox.StatusToStr(
  const DropboxStatus: TDropboxStatus): string;
type
  StatusStrArray = array[TDropboxStatus] of string;
const
  DropboxStatusString: StatusStrArray = (
    'Pipe Error', 'Dropbox not running', 'Not in Dropbox',
    'Up to Date', 'Sync in Progress', 'Sync Problem'
  );
begin
  Result := DropboxStatusString[DropboxStatus];
end;

end.
